#include "dialog.h"
#include <gst/gst.h>

static gboolean bus_call (GstBus     *bus,
        GstMessage *msg,
        gpointer    data);

Dialog::Dialog(QWidget* parent):pipeline(0)
{ 
    setupUi(this); 
}

void Dialog::on_gstLaunchButton_clicked()
{
    g_print("clicked %p", pthread_self() );

    // init
    gst_init(0,0);
    textEdit->append("gst_init\n");

    // create the pipeline through gst parse
    this->pipeline = gst_parse_launch (lineEdit->text().toLatin1(), NULL);

    if (!pipeline )
    {
        textEdit->append("pipeline creation FAIL\n");
        return;
    }
    textEdit->append("pipeline created \n");



    /* we add a message handler */
    GstBus* bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
    gst_bus_add_watch (bus, bus_call, this);
    gst_object_unref (bus);

    /* Move to playing state */
    textEdit->append("playing... \n");
    gst_element_set_state (pipeline, GST_STATE_PLAYING);

}
#ifdef USE_EVENT
bool Dialog::event ( QEvent * event )
{
    if ( event->type() == QEvent::User )
    {
        g_print(" received event %d thread %p", event->type(), pthread_self() );
        return TRUE;
    }
    return TRUE;
}
#endif // USE_EVENT

void Dialog::processGstMsg(GstMessage* msg)
{
    switch (GST_MESSAGE_TYPE (msg)) {

        case GST_MESSAGE_EOS:

            /* Out of the main loop, clean up nicely */
            g_print ("Returned, stopping playback\n");
            gst_element_set_state (pipeline, GST_STATE_NULL);

            g_print ("Deleting pipeline\n");
            gst_object_unref (GST_OBJECT (pipeline));
            break;
    }

}

static gboolean
bus_call (GstBus     *bus,
        GstMessage *msg,
        gpointer    data)
{
    Dialog* gstDia = (Dialog*)data;


    switch (GST_MESSAGE_TYPE (msg)) {

        case GST_MESSAGE_EOS:

            gstDia->appendMsg("End of stream\n");
            break;

        case GST_MESSAGE_ERROR: 
            {
                gchar  *debug;
                GError *error;

                gst_message_parse_error (msg, &error, &debug);
                g_free (debug);

                g_printerr ("Error: %s\n", error->message);
                gstDia->appendMsg(QString("Error: ") + error->message);
                g_error_free (error);

                break;
            }
        default:
            break;
    }

#ifdef USE_EVENT
    QApplication::postEvent (gstDia, new QEvent(QEvent::User) );
#else
    gstDia->processGstMsg(msg);
#endif // USE_EVENT

    g_print("}bus exit");

    return TRUE;
}

